function [sys,x0,str,ts] = fcc_ekf(t,x,u,flag)
% fcc_ekf is an S-function which implement extended kalman filter for fcc
% Rf:
% [1] Gauthier-Kupka
% [2] Busvelle-Gauthier

%
% Dispatch the flag.
%
switch flag,
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes; % Initialization
  case 1
    sys = mdlDerivatives(t,x,u); % Calculate derivatives
  case 3
    sys = mdlOutputs(t,x,u); % Calculate outputs  
  case { 2, 4, 9 } % Unused flags
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); % Error handling
end
% End of csfunc.
%==============================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the 
% S-function.
%==============================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes()
%
% Call simsizes for a sizes structure, fill it in and convert it 
% to a sizes array.
%
global LastCrcSaturation_ LastCcatSaturation_
LastCrcSaturation_=[];
LastCcatSaturation_=[];

sizes = simsizes;
sizes.NumContStates  = 44;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 9;
sizes.NumInputs      = 5;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);
%
% Initialize the initial conditions.
%
P0=zeros(8,8);
x0=[775;943;0.3;1.2;0;2.5066;0;0];
x0=abs(x0+0.1*randn(size(x0)));
%disp('Observateur bien initialis non corrig!!!'),
x0=[x0;SymReshape(P0)];
%
% str is an empty matrix.
%
str = [];
%
% Initialize the array of sample times; in this example the sample 
% time is continuous, so set ts to 0 and its offset to 0.
%
ts = [0 0];
% End of mdlInitializeSizes.
%
%==============================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%==============================================================
function sys = mdlDerivatives(t,x,u)
% NAMES / DESCRIPTION / UNITS
%   R     Ideal gas constant J/mol.K
%   H     Holdup kg
%   R     Rate kg/s
%   S     Specific heat J/kg.K
%   P     Pressure kPa
%   T     Temperature K
%   D     Heat J/kg
%   A     Activation energy J/mol
%   F     factor, without unit
%   k     empirical constant rates /kPa.s
global LastCrcSaturation_ LastCcatSaturation_
R = 8.314;
%
% Reactor
% -------
Hra = 1.85e4;
Pra = 211.7;
DHcr = 4.65e5;
% feed
Rtf = 41;
Ttf = 492.8;
Stf = 3140;
% Reaction 1
Acr = 6.28e4;
kcr = 8.31e-2;
% Reaction 2
Acc = 4.18e4;
kcc = 2.66e-4;
% Regenerator
% -----------
Hrg = 1.53e5;
Prg = 254.4;
DHfv = 1.74e5;
DHrg = 3.02e7;
% Air
Rai = 26;
Tai = 394;
Sai = 1130;
% Catalyst
Rc  = 290;
Sc  = 1047;
% Reaction (02)
Aor = 1.47e5;
%
Tra = x(1);
Trg = x(2);
Crc = x(3);
Csc = x(4);
Fcf = x(5);
Rcb = x(6);
dRcb = x(7);
d2Rcb = x(8);
P=SymReshape(x(9:44));
Rai = u(1);
dRai = u(2);
Tra_mes = u(3);
Trg_mes = u(4);
theta = u(5);

if isempty(LastCrcSaturation_),LastCrcSaturation_=0;end,
if isempty(LastCcatSaturation_),LastCcatSaturation_=0;end,

if Crc<=1e-3,
    Crc=1e-3;
    if t>LastCrcSaturation_+5/60,
        disp(['Crc saturation at t = ',num2str(t)]),
        LastCrcSaturation_=t;
    end,
end,

Ccat2=100*Pra*Hra*kcc*exp( -Acc/(R*Tra) ) / (Rc*Crc^0.06);
if Ccat2>1e-4,
    Ccat=sqrt(Ccat2);
else
    Ccat=1e-2;
    if t>LastCcatSaturation_+5/60,
        disp(['Ccat saturation at t = ',num2str(t)]),
        LastCcatSaturation_=t;
    end,
end,

Kcr = kcr/(Ccat*Crc^0.15)*exp( -Acr/(R*Tra) );
Rcr = Kcr*Pra*Hra;
Ctf = 1 / ( 1 + Rtf/Rcr);
dTra = ( Sc*Rc*(Trg-Tra) + Stf*Rtf*(Ttf-Tra) - Rtf*DHfv - Rtf*Ctf*DHcr ) / (Sc*Hra);

%Kor = 1.16e-5*exp( Aor/R*( 1/866.7 - 1/Trg ) );
%Kod = 6.34e-9*Rai^2;
%Ofg = 21*exp( - (Prg*Hrg/Rai) / ( 1/Kod + 1/(Kor*Crc) ) );
%Rcb = Rai*(21-Ofg)/242;
dTrg = ( Sc*Rc*(Tra-Trg) + Sai*Rai*(Tai-Trg) + Rcb*DHrg ) / (Sc*Hrg);

dCrc = ( Rc*(Csc-Crc) - 100*Rcb )/Hrg;

Kcc = kcc/(Ccat*Crc^0.06)*exp( -Acc/(R*Tra) );
Rcc = Kcc*Pra*Hra;
Rad = Fcf*Rtf;
Rcf = Rcc + Rad;
dCsc = ( Rc*(Crc-Csc) + 100*Rcf )/Hra;

dFcf = 0;
d3Rcb = 0;

C=[eye(2),zeros(2,6)];
% The following line has been tested and works well
%Q=theta*diag([1,1,0.1,0.1,0.001,0.5,0.3,0.1])/100000; % ATTENTION: PROVISOIRE
% Now, another solution more theoretically justified
DeltaInv=diag([1,1,theta,theta^2,theta^3,theta,theta^2,theta^3]);
%Q=diag([1,1,0.1,0.1,0.001,1e-4,1e-4,1e-4])/1e8;
%Q=diag([1,1,0.1,0.1,0.001,1e-2,1e-2,1e-2])/1e8;

Q = diag([1,1,0.1,0.1,0.001,0,0,0])/1e8;   % provient du tuning de l'observateur
Q = Q + diag([0,0,0,0,0,1e-2,1e-4,1e-6])/1e8;   % nouvelle partie de l'tat

Q=theta^2*DeltaInv*Q*DeltaInv;
%a=Rtf*Pra*Hra/(Rcr+Rtf)^2*(Kcr/Tra^2);
%b=Rtf*Pra*Hra/(Rcr+Rtf)^2*(-0.06*Kcr/Crc);
%DPhiInv=[1,0,0,0,0,0,0,0;...
%         0,1,0,0,0,0,0,0;...
%         -a/b,0,1/b,0,0,0,0,0;...
%         -(a*Csc)/(b*Crc),0,Csc/(b*Crc),Crc,0,0,0,0;...
%         -(a*Fcf)/(b*Crc),0,Fcf/(b*Crc),0,Crc,0,0,0;...
%         0,0,0,0,0,1,0,0;...
%         0,0,0,0,0,0,1,0;...
%         0,0,0,0,0,0,0,1];
%Q=DPhiInv*Q*DPhiInv';
% end
R1=eye(2);

xdot = [dTra;dTrg;dCrc;dCsc;dFcf;dRcb;d2Rcb;d3Rcb] + P*C'*R1*[Tra_mes-Tra;Trg_mes-Trg];
%
% Jacobien
% --------
A=zeros(8,8);
% Equation Tra
dRcr_dCrc = -0.12*Rcr/Crc;
dRcr_dTra = (Acr-Acc/2)/(R*Tra^2)*Rcr;
dCtf_dRcr = Rtf/(Rcr+Rtf)^2;
dCtf_dCrc = dCtf_dRcr * dRcr_dCrc;
dCtf_dTra = dCtf_dRcr * dRcr_dTra;
A(1,1) = ( -Sc*Rc -Stf*Rtf -Rtf*DHcr*dCtf_dTra) / (Sc*Hra);
A(1,2) = Rc/Hra;
A(1,3) = ( -Rtf*DHcr*dCtf_dCrc) / (Sc*Hra);
% Equation Trg
A(2,1) = Rc/Hrg;
A(2,2) = ( -Sc*Rc -Sai*Rai) / (Sc*Hrg);
A(2,6) = DHrg / (Sc*Hrg);
% Equation Crc
A(3,3) = -Rc/Hrg;
A(3,4) = Rc/Hrg;
A(3,6) = -100/Hrg;
% Equation Csc
dRcf_dTra = Acc/(2*R*Tra^2)*Rcf;
dRcf_dCrc = -0.03/Crc*Rcf;
dRcf_dFcf = Rtf;
A(4,1) = 100*dRcf_dTra/Hra;
A(4,3) = (Rc+100*dRcf_dCrc)/Hra;
A(4,4) = -Rc/Hra;
A(4,5) = 100*dRcf_dFcf/Hra;
% Equations Fcf (suppos constant) et Rcb (modle local du 2nd degr)
A(6,7) = 1;
A(7,8) = 1;

if t==0,
    save A.mat A
    disp('Jacobian matrix saved as A in A.mat'),
end,

Pdot=A*P+P*A'+Q-P*C'*R1*C*P;
%Pdot=zeros(8,8);

syscpx = 3600*[xdot;SymReshape(Pdot)];

if isreal(syscpx) & (length(syscpx)==44),
    sys = 3600*[xdot;SymReshape(Pdot)];
else,
    sys = 3600*[xdot;SymReshape(Pdot)];
    disp(syscpx),
end,
    

% End of mdlDerivatives.
%
%==============================================================
% mdlOutputs
% Return the block outputs.
%==============================================================
%
function sys = mdlOutputs(t,x,u)
Hrg = 1.53e5;
Prg = 254.4;
Tra = x(1);
Trg = x(2);
Crc = x(3);
Csc = x(4);
Fcf = x(5);
Rcb = x(6);
Rai = u(1);
Tra_mes = u(3);
Trg_mes = u(4);

Kod = 6.34e-9*Rai^2;
Ofg = 21-242*Rcb/Rai;
Kor=Ofg2Kor(Ofg,Rai,Crc);

InnovError=abs(Tra-Tra_mes)/Tra_mes+abs(Trg-Trg_mes)/Tra_mes;

pmax=max(abs(x(9:44))); % following Krener's Idea

%disp([x(7),x(8)])

sys = [Tra;Trg;Crc;Csc;Fcf;Rcb;Kor;InnovError;pmax];
% End of mdlOutputs.