% static model
Tra = x(1);
Trg = x(2);
Crc = x(3);
Csc = x(4);
Fcf = x(5);
Rcb = x(6);
dRcb = x(7);
d2Rcb = x(8);

R = 8.314;
%
% Reactor
% -------
Hra = 1.85e4;
Pra = 211.7;
DHcr = 4.65e5;
% feed
Rtf = 41;
Ttf = 492.8;
Stf = 3140;
% Reaction 1
Acr = 6.28e4;
kcr = 8.31e-2;
% Reaction 2
Acc = 4.18e4;
kcc = 2.66e-4;
% Regenerator
% -----------
Hrg = 1.53e5;
Prg = 254.4;
DHfv = 1.74e5;
DHrg = 3.02e7;
% Air
Rai = 26;
Tai = 394;
Sai = 1130;
% Catalyst
Rc  = 290;
Sc  = 1047;
% Reaction (02)
Aor = 1.47e5;

if Crc<=1e-3,
    Crc=1e-3;
    disp(['Crc saturation at t = ',num2str(t)]),
end,

Ccat2=100*Pra*Hra*kcc*exp( -Acc/(R*Tra) ) / (Rc*Crc^0.06);
if Ccat2>0,
    Ccat=sqrt(Ccat2);
else
    Ccat=0;
    disp(['Ccat saturation at t = ',num2str(t)]),
end,

Kcr = kcr/(Ccat*Crc^0.15)*exp( -Acr/(R*Tra) );
Rcr = Kcr*Pra*Hra;
Ctf = 1 / ( 1 + Rtf/Rcr);
dTra = ( Sc*Rc*(Trg-Tra) + Stf*Rtf*(Ttf-Tra) - Rtf*DHfv - Rtf*Ctf*DHcr ) / (Sc*Hra);

%Kor = 1.16e-5*exp( Aor/R*( 1/866.7 - 1/Trg ) );
%Kod = 6.34e-9*Rai^2;
%Ofg = 21*exp( - (Prg*Hrg/Rai) / ( 1/Kod + 1/(Kor*Crc) ) );
%Rcb = Rai*(21-Ofg)/242;
dTrg = ( Sc*Rc*(Tra-Trg) + Sai*Rai*(Tai-Trg) + Rcb*DHrg ) / (Sc*Hrg);

dCrc = ( Rc*(Csc-Crc) - 100*Rcb )/Hrg;

Kcc = kcc/(Ccat*Crc^0.06)*exp( -Acc/(R*Tra) );
Rcc = Kcc*Pra*Hra;
Rad = Fcf*Rtf;
Rcf = Rcc + Rad;
dCsc = ( Rc*(Crc-Csc) + 100*Rcf )/Hra;

dFcf = 0;
d3Rcb = 0;

C=[eye(2),zeros(2,6)];
Q=diag([1,1,0.1,0.1,0.001,0.5,0.3,0.1])/100000;
R1=eye(2);

xdot = [dTra;dTrg;dCrc;dCsc;dFcf;dRcb;d2Rcb;d3Rcb];
%
% Jacobien
% --------
A=zeros(8,8);
% Equation Tra
dRcr_dCrc = -0.12*Rcr/Crc;
dRcr_dTra = (Acr-Acc/2)/(R*Tra^2)*Rcr;
dCtf_dRcr = Rtf/(Rcr+Rtf)^2;
dCtf_dCrc = dCtf_dRcr * dRcr_dCrc;
dCtf_dTra = dCtf_dRcr * dRcr_dTra;
A(1,1) = ( -Sc*Rc -Stf*Rtf -Rtf*DHcr*dCtf_dTra) / (Sc*Hra);
A(1,2) = Rc/Hra;
A(1,3) = ( -Rtf*DHcr*dCtf_dCrc) / (Sc*Hra);
% Equation Trg
A(2,1) = Rc/Hrg;
A(2,2) = ( -Sc*Rc -Sai*Rai) / (Sc*Hrg);
A(2,6) = DHrg / (Sc*Hrg);
% Equation Crc
A(3,3) = -Rc/Hrg;
A(3,4) = Rc/Hrg;
A(3,6) = -100/Hrg;
% Equation Csc
dRcf_dTra = Acc/(2*R*Tra^2)*Rcf;
dRcf_dCrc = -0.03/Crc*Rcf;
dRcf_dFcf = Rtf;
A(4,1) = 100*dRcf_dTra/Hra;
A(4,3) = (Rc+100*dRcf_dCrc)/Hra;
A(4,4) = -Rc/Hra;
A(4,5) = 100*dRcf_dFcf/Hra;
% Equations Fcf (suppos constant) et Rcb (modle local du 2nd degr)
A(6,7) = 1;
A(7,8) = 1;
