function [sys,x0,str,ts] = xyplotting(t,x,u,flag)
% CSFUNC An example M-file S-function for defining a system of
% continuous state equations:
%      x' = Ax + Bu
%      y  = Cx + Du
%
% Generate a continuous linear system:
h=figure(1);
%
% Dispatch the flag.
%
switch flag,
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes(h); % Initialization
  case 2
    sys = mdlUpdate(t,x,u,h);
  case { 1, 3, 4, 9 } % Unused flags
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); % Error handling
end
% End of csfunc.
%==============================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the 
% S-function.
%==============================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(h)
%
% Call simsizes for a sizes structure, fill it in and convert it 
% to a sizes array.
%
sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 2;
sizes.NumOutputs     = 0;
sizes.NumInputs      = 2;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);
%
% Initialize the initial conditions.
%
x0 = [0;0];
str = [];
ts = [0.1 0];
%
% Plot initial function
%
set(h,'name','02 reaction rate constant versus regenerator temperature');
clf;
R = 8.314;
Aor = 1.47e5;
Trg=970:0.1:1010;
Kor = 1.16e-5*exp( Aor/R*( 1/866.7 - 1./Trg ) );
plot(Trg,Kor,'r-','LineWidth',2),hold on,
%axis([970,1010,1e-4,2e-4]),
axis([970,1010,0e-4,3e-4]),
plot(0,0,'b--','LineWidth',2);
%text(Trg(360),Kor(360),'Actual 0_{2} reaction rate \rightarrow',...
%     'HorizontalAlignment','right'),
%legend('Actual Kor','Estimated Kor',4),
%xlabel('T_{rg} (K)'),ylabel('K_{or} (s^{-1}.kPa^{-1})'),
p=get(h,'CurrentAxes');
set(p,'YTick',[0,1e-4,2e-4,3e-4]),
set(p,'YTickLabel',[0,1e-4,2e-4,3e-4]),
set(p,'XTick',[970,980,990,1000,1010]),
set(p,'XTickLabel',[970,980,990,1000,1010]),
set(p,'NextPlot','Add'); % same as hold on
set(p,'XLimMode','manual');
set(p,'YLimMode','manual');
% End of mdlInitializeSizes.
%
%==============================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%==============================================================
function sys = mdlUpdate(t,x,u,h)
p=get(h,'CurrentAxes');pp=get(p,'Children');
for i=1:length(pp),
    if isequal(get(pp(i),'type'),'line'),
        ms=get(pp(i),'MarkerSize');
        if ms>1,
            set(pp(i),'MarkerSize',ms-1);
        end,
    end,
end,
plot([x(1),u(1)],[x(2),u(2)],'--bs','LineWidth',2,...
                'Marker','o',...
                'MarkerEdgeColor','k',...
                'MarkerFaceColor','g',...
                'MarkerSize',8)

tint=round(t);
if abs(t-tint)<1e-8,
    s=['0',int2str(tint)];
    l=length(s);
    s=s((l-1):l);
    fn=['KorTrg',s,'.eps'];
    print(h,'-depsc','-loose',fn),
end,
            
sys = [u(1);u(2)];
% End of mdlUpdate.
%
%==============================================================
% mdlOutputs
% Return the block outputs.
%==============================================================
%
%function sys = mdlOutputs(t,x,u,A,B,C,D)
%sys = [];
% End of mdlOutputs.